' VBScript to Add a Single Entity
'   lines starting with an apostrophe are comments

' create Visual CADD and point objects
set vcadd = CreateObject("VisualCADD.Application.7")
set pt0 = CreateObject("VisualCADD.Point2D.7")
set pt1 = CreateObject("VisualCADD.Point2D.7")

' Optional - Set properties for the new entity:
'      While all the properties can be modified after the entity 
'      has been added to the database, it is convenient to set 
'      the properties prior to the operation.
'      The properties for entities typically include layer, 
'      color, line type, and line width.  These can be set with 
'      the following API calls.
'      If you do not set a specific property, Visual CADD will 
'      use the current properties when creating the entity.
'      Compound entities such as dimensions, text, fills, and
'      hatches require specific settings for the respective 
'      entity type.
'      The following code sets the layer index to 5, 
'      the color property to blue (index 9), 
'      the line type property to SOLID (index 0),
'      and the line width property index to 1 (pixel).

vcadd.SetLayerIndex(5)
vcadd.SetColorIndex(9)
vcadd.SetLineTypeIndex(0)
vcadd.SetLineWidthIndex(1)

'  Add the entity to the database:
'      To add the entity to the drawing, specify the point(s)
'      location which can be determined either directly through 
'      code or as a result of user input (demonstrated in 
'      another topic).  The point(s) are then passed to the add
'      command to place the entity.
'      Note each entity requires a different number of input 
'      points to define the geometry. For example, a line 
'      entity requires two point locations for the end points 
'      while an elliptical arc requires seven points to define
'      the placement. Verify the number of points and their 
'      purpose in the Alphabetical Listing of API 
'      Calls.
'      The following code adds a line from (0.0, 0.0) to 
'      (10.0, 10.0) based on values established through 
'      code. Note the points have been defined as a Point2D
'      and contain an x and y value.
'      The -1 means to place the entity in the drawing, other 
'      options can place it in a symbol definition.

pt0.X = 0.0
pt0.Y = 0.0
pt1.X = 10.0
pt1.Y = 10.0
vcadd.AddLineEntity -1, pt0, pt1

'  Optional - Make the new entity the current entity:
'      A new entity is always appended to the end of the 
'      database and given an entity handle (index) one greater 
'      than the previous last entity.  It may be necessary to 
'      make the new entity current for tracking its ID, changing
'      properties, redrawing to the screen, adding data to the 
'      entity, or other operations.  Each entity in the drawing
'      database is referenced by an entity handle (index) 
'      allowing applications to quickly access entities with 
'      VCSetCurrentEntity.
'      The following code sets the newly placed entity as the 
'      current entity.

hEnt = vcadd.LastEntity()
vcadd.SetCurrentEntity(hEnt)

'  Optional - Draw the new entity to the screen:
'      New entities are not automatically displayed on the 
'      screen after being added to the database.  The entity 
'      will be drawn during the next redraw or paint event,
'      such as a VCZoomAll() or a UI command Regen/RD.  In order
'      to draw the entity immediately, use VCDrawCurrentEntity:

vcadd.DrawCurrentEntity()

' release Visual CADD and point objects
set vcadd = Nothing
set pt0 = Nothing
set pt1 = Nothing
