' VBScript to Add Arc Text
'   lines starting with an apostrophe are comments

' create Visual CADD object and connect to events
Set vcadd = WScript.CreateObject("VisualCADD.Application.7")
WScript.ConnectObject vcadd, "vcadd_"

' create point objects and variables
Set ptCenter  = CreateObject("VisualCADD.Point2D.7")
Set ptRadius = CreateObject("VisualCADD.Point2D.7")
Set pt0 = CreateObject("VisualCADD.Point2D.7")
Set pt1 = CreateObject("VisualCADD.Point2D.7")

Const Pi = 3.14159265358979
Dim dRadius
Dim dAngle
Dim hEntRubber
hEntRubber = -1

'   ArcText tool will draw text curved along an arc entered 
'   by the user.  The tool has 3 states:
'       (1) place center point of the arc
'       (2) place radius and text start point
'       (3) type text
'   ArcText supports the BackSpace key and undo/redo.
'   ArcText is placed as a group of individual characters.

'   Optional - save the settings to be restored later
vcadd.SaveSettings()

'   Increment the undo level to support undo/redo.
'   Optional - use VCBeginOperation() instead.  
'       VCBeginOperation() does only two things - it 
'       increments the undo level and saves the current end 
'       of the database for cleanup when done.  The ArcText 
'       tool uses a rubberband which is appended at the end
'       of the database, so we can use that entity handle for 
'       cleanup instead.
vcadd.IncrementUndoLevel()

'   Start a user tool and define the status bar command line
'   prompts for the ArcText tool.  The states are 0, 1, 2.
'   The first prompt for state 0 is set by VCSetUserTool()
'   and the next two prompts are set by VCSetPrompt().
vcadd.SetUserTool 2, "ArcText", "Enter Center Point"
vcadd.SetPrompt 1, "Enter Start Point of Text"
vcadd.SetPrompt 2, "Enter Text, press [Back] key to re-type, [Enter] key to finish, [Esc] key to abort"

'   Set alert events so our tool gets notifications from 
'   Visual CADD about user interface events, other tool
'   operations, and Windows messages.  We are requesting 
'   alerts on mouse-down (point placement), mouse-move (for
'   rubberbands), pen up (when tool ends normally),
'   abort (when user aborts or another tool starts), and
'   Windows messages.  Each of these has a corresponding 
'   callback function below.
vcadd.SetAlertEvent &h01 + &h02 + &h04 + &h20, &h01

'   Loop to keep the script running while we wait for the user input
tfLoop = True
While tfLoop = True
    '   Brief sleep to free the proceesor.  ArcText still gets events 
    '   during the sleep
    Wscript.Sleep 100
Wend

Set VCadd = Nothing
Set ptCenter = Nothing
Set ptRadius = Nothing
Set pt0 = Nothing
Set pt1 = Nothing

Function vcadd_PreTranslateGetMessage(ByRef hWnd, ByRef uMsg, ByRef wParam, ByRef lParam, ByRef Time, ByRef PointX, ByRef PointY )
    '   PreTranslateGetMessage processes posted messages, usually
    '   keyboard messages, but any posted messages may be 
    '   processed.

    '   The function return will tell Visual CADD if we have 
    '   processed the message and whether it should process it.
    '   The default is FALSE, we have not processed it.  Setting 
    '   to TRUE tells Visual CADD to not process the message.

    '   The default return, we have not processed the message
    vcadd_PreTranslateGetMessage = 0

    '   get the tool state so we know the state of the tool
    Dim iState
    iState = vcadd.GetUserToolState()

    '   &H100 is the WM_KEYDOWN and 27 is VK_ESCAPE or Esc key.
    '   Abort the ArcText tool anytime the user presses Esc.
    If ((uMsg = &H100) And (wParam = 27)) Then
        vcadd.Abort()
        vcadd_PreTranslateGetMessage = 1
    End If

    '   Once the tool reaches state 2, we will process keyboard
    '   input as the user types.
    If (iState = 2) Then
        If (uMsg = &H100) Then
            '   &H100 is WM_KEYDOWN
            If (wParam = 13) Then
                '   13 is VK_RETURN or Enter key.
                '   Calling VCPenUp() ends a tool normally.
                vcadd.PenUp()
                vcadd_PreTranslateGetMessage = 1
            ElseIf (wParam = 8) Then
                '   8 is VK_BACK or backspace key.
                '   Our BackSpace function will remove the last
                '   typed character.
                BackSpace()
                vcadd_PreTranslateGetMessage = 1
            End If
        ElseIf (uMsg = &H102) Then
            '   &H102 is WM_CHAR
            If (wParam >= 32) Then
                '   characters 32 and above are printable 
                '   so we add them to our ArcText by calling our 
                '   TextIt function.
                TextIt(Chr(wParam))
            End If
            vcadd_PreTranslateGetMessage = 1
        End If
    End If

End Function

Sub vcadd_MouseDown(wParam, lParam)
    '   get the tool state so we know the state of the tool
    Dim iState 
    iState = vcadd.GetUserToolState()

    '   Visual CADD increases the tool state of user tools after 
    '   mouse clicks.  Tool state 1 means the tool state was 
    '   previously 0 and the user has clicked for the center of
    '   the arc.     
    If (iState = 1) Then
        '   get the mouse-click point from Visual CADD
        Set ptCenter = vcadd.GetUserToolLBDown()

        '   add circle and line (radius) entities to be the 
        '   rubberband.  Save the rubberband entity handle for
        '   later use.
        vcadd.AddCircleEntity -1, ptCenter, ptCenter
        hEntRubber = vcadd.LastEntity()
        vcadd.AddLineEntity -1, ptCenter, ptCenter
    End If

    '   Tool state 2 means the tool state was previously 1 and 
    '   the user has clicked for the start of the text.
    If (iState = 2) Then
        '   get the mouse-click point from Visual CADD
        Set ptRadius = vcadd.GetUserToolLBDown()

        '   calculate the radius distance and the angle.
        Dim dX 
        Dim dY 

        dX = ptRadius.X - ptCenter.X
        dY = ptRadius.Y - ptCenter.Y

        dRadius = Sqr(dX * dX + dY * dY)
        dAngle = Atan2(dY, dX)
    End If

    '   Tool state 3 means the tool state was previously 2 and 
    '   the user has clicked to finish the tool.
    If (iState = 3) Then
        vcadd.PenUp()
    End If

End Sub

Sub vcadd_MouseMove(wParam, lParam )
    '   get the tool state so we know the state of the tool
    Dim iState 
    iState = vcadd.GetUserToolState()

    '   use the mouse move to draw the rubberband while the user
    '   is choosing the arc start point.  Tool state 1 means the 
    '   user has already clicked the center so we will draw the
    '   rubberband to the radius at the current mouse location.
    If (iState = 1) Then
        '   drawing rubberbands involves erasing the old 
        '   rubberband and drawing the new one.  The function
        '   VCDrawCurrentEntityXOR() is used to draw entities 
        '   using the user's rubberband color.  The XOR means 
        '   if it is already drawn on-screen XOR will remove it; 
        '   if it is not already drawn on-screen XOR will draw it.

        '   erase the circle and line (radius) rubberbands
        vcadd.SetCurrentEntity(hEntRubber)
        vcadd.DrawCurrentEntityXOR()

        vcadd.SetCurrentEntity(hEntRubber + 1)
        vcadd.DrawCurrentEntityXOR()

        '   get the mouse location point from Visual CADD
        Set ptRadius = vcadd.GetUserToolMouseMove()

        '   set the new point in the circle and line (radius) 
        '   rubberband entities and draw them
        vcadd.SetCurrentEntity(hEntRubber)
        vcadd.SetCurrentEntityPoint 1, ptRadius
        vcadd.DrawCurrentEntityXOR()

        vcadd.SetCurrentEntity(hEntRubber + 1)
        vcadd.SetCurrentEntityPoint 1, ptRadius
        vcadd.DrawCurrentEntityXOR()
    End If

End Sub

Sub TextIt(strText)
    '   TextIt is called by the PreTranslateGetMessage after we 
    '   are in state 2, meaning we are accepting text input.

    '   create variables for calculation 
    Dim strChar 
    Dim hEntLast 
    Dim iLen 
    Dim i 
    Dim dSpan 
    Dim dWidth 

    '   We are using the TextIt function to process one character
    '   at a time.  However, the function can also process full 
    '   length text strings.  We get the length of the string, 
    '   1 in the case of the current ArcText tool.
    iLen = Len(strText)

    '   Loop through the string one character at a time
    For i = 1 To iLen
        '   get a single character and set Visual CADD text to 
        '   use that character
        strChar = Mid(strText, i, 1)
        vcadd.SetTextString(strChar)

        '   Total character width is the width of the text entity
        '   using that character plus character spacing
        dWidth = GetStringWidth()

        '   Calculate the text placement point from the center,
        '   radius, and angle
        pt0.X = ptCenter.X + dRadius * Cos(dAngle)
        pt0.Y = ptCenter.Y + dRadius * Sin(dAngle)

        '   Calculate how much angle is spanned by the character
        '   width
        dSpan = Atan2(dWidth, dRadius)

        '   The standard angle in Visual CADD is measured 
        '   counter-clockwise from 3 o'clock whereas text 
        '   rotation angle of 0.0 means upright text, or 12 
        '   o'clock, a difference of 90 degrees.  The API uses 
        '   radians for angles, so 90 degrees is pi / 2 and we 
        '   adjust the text rotation angle by that amount.
        vcadd.SetTextRot(dAngle - dSpan / 2.0 - Pi / 2.0)

        '   spacing requires left-justified text
        vcadd.SetTextJustify(Asc("L"))

        '   Add the text at the calculated point
        vcadd.AddTextEntity -1, pt0

        '   Adjust the angle clockwise for the next character
        dAngle = dAngle - dSpan

        '   Draw the newly placed character
        hEntLast = vcadd.LastEntity()
        vcadd.SetCurrentEntity(hEntLast)
        vcadd.DrawCurrentEntity()
    Next

End Sub

Function GetCharacterSpace() 
    '   Get the character space setting so we can calculate
    '   how to space the text around the arc.  
    If (vcadd.GetTextFontType() = 1) Then
        '   Font type 1 is a vector font.  The character spacing
        '   is a percentage of the text height.
        Dim dHeight 
        Dim dCharSpPerCent 
        dHeight = vcadd.GetTextHeight()
        dCharSpPerCent = vcadd.GetTextCharSpace()
        GetCharacterSpace = dHeight * dCharSpPerCent
    Else
        '   Font type 0 is a TrueType font.  The character 
        '   spacing is built directly into TrueType character 
        '   definitions so there is no additional spacing.
        GetCharacterSpace = 0.0
    End If

End Function

Function GetStringWidth() 
    '   use rotation angle 0.0 to place a temporary text 
    '   entity we can use to find the width of the character.
    vcadd.SetTextRot(0.0)
    vcadd.AddTextEntity -1, ptCenter

    '   VCEntityExtents() gets the size of an entity.  After
    '   we get the size, we can use VCTruncFrom() to delete
    '   this temporary entity.
    Dim hEntLast 
    hEntLast = vcadd.LastEntity()
    vcadd.EntityExtents hEntLast, pt0, pt1
    vcadd.TruncFrom(hEntLast)

    '   Total character width is the width of the text entity
    '   using that character plus character spacing
    GetStringWidth = pt1.X - pt0.X + GetCharacterSpace()

End Function

Sub BackSpace()
    '   BackSpace is called by the PreTranslateGetMessage after
    '   we are in state 2, meaning we are accepting text input,
    '   and the user has pressed the Back key.

    '   create variables for calculation 
    Dim hEntLast 
    Dim dSpan 
    Dim dWidth 

    '   Get the last entity handle
    hEntLast = vcadd.LastEntity()

    '   Before processing a backspace key, be sure our text 
    '   string is not already empty.  The rubberband is two
    '   entities, so the last entity must be after the second
    '   rubberband entity.
    If (hEntLast > hEntRubber + 1) Then

        '   Match the last entity so we can use its text string, 
        '   actually an individual character
        vcadd.SetCurrentEntity(hEntLast)
        vcadd.MatchCurrentEntity()

        '   Total character width is the width of the text entity
        '   using that character plus character spacing
        dWidth = GetStringWidth()

        '   Calculate how much angle is spanned by the character
        '   width and adjust the angle counter-clockwise back 
        '   to the previous character.
        dSpan = Atan2(dWidth, dRadius)
        dAngle = dAngle + dSpan

        '   Erase the last entity, draw it so it erases from the
        '   screen, and truncate the database to permanently 
        '   remove it.
        hEntLast = vcadd.LastEntity()
        vcadd.SetCurrentEntity(hEntLast)
        vcadd.SetCurrentErased()
        vcadd.DrawCurrentEntity()
        vcadd.TruncFrom(hEntLast)
    End If

End Sub

Sub vcadd_PenUp(wParam, lParam )
    '   ArcText tool has completed normally, so we need to 
    '   cleanup the database and do final processing.

    '   Get the last entity handle
    Dim hEntLast 
    hEntLast = vcadd.LastEntity()

    '   Check that the user has entered some text, not just
    '   pressed Enter.
    If ((hEntRubber = -1) Or (hEntLast < hEntRubber + 2)) Then
        '   There was no text entered, so Abort instead.
        vcadd.Abort()
    Else
        '   Call the CleanUp function to handle routine 
        '   processing
        CleanUp()

        '   clear selection
        vcadd.ClearSelection()

        '   Loop through the text entities and select them
        For hEnt = hEntRubber + 2 To hEntLast
            vcadd.SetCurrentEntity(hEnt)
            vcadd.SetCurrentSelected()
        Next

        '   Purge the database to cleanup redo conflicts possibly
        '   left by previous tools.  Optional - if you used 
        '   VCBeginOperation(), then you should call 
        '   VCEndOperation() which does the same VCPurgeRedo().
        vcadd.PurgeRedo()

        '   Group the ArcText entities so they stay together
        '   when selected
        vcadd.GroupSelected()
    End If

    '   Loop can stop running
    tfLoop = False

End Sub

Sub vcadd_Abort(wParam, lParam )
    '   ArcText tool has aborted, so we need to cleanup the 
    '   database.

    '   Call the CleanUp function to handle routine 
    '   processing
    CleanUp()

    '   If ArcText made it as far as a rubberband, then erase
    '   from the screen any text.
    If (hEntRubber <> -1) Then
        '   Get the last entity handle
        Dim hEntLast 
        hEntLast = vcadd.LastEntity()

        '   Loop through the text entities
        For hEnt = hEntRubber + 2 To hEntLast
            '   Erase each text entity and draw it on-screen
            '   to erase it from screen
            vcadd.SetCurrentEntity(hEnt)
            vcadd.SetCurrentErased()
            vcadd.DrawCurrentEntity()
        Next

        '   Truncate the database starting with the rubberband
        '   to restore the database to its original condition.
        vcadd.TruncFrom(hEntRubber)
    End If

    '   Decrement the undo level since the ArcText was aborted 
    '   and no entities were added.  Optional - if you used 
    '   VCBeginOperation(), then you should call 
    '   VCAbortOperation() which does the same VCTruncFrom() and 
    '   VCDecrementUndoLevel().
    vcadd.DecrementUndoLevel()

    '   Loop can stop running
    tfLoop = False

End Sub

Sub CleanUp()
    '   If ArcText made it as far as a rubberband, then erase
    '   from the screen the two rubberband entities.  CleanUp is
    '   called by both ToolComplete and ToolAbort, so we do not
    '   know whether the database will be purged or truncated.
    '   Thus we use VCSetCurrentEntityPurged() to both erase the
    '   entities and mark them for purging.  Draw the erased
    '   entities to remove them from the screen.
    If hEntRubber <> -1 Then
        vcadd.SetCurrentEntity(hEntRubber)
        vcadd.SetCurrentEntityPurged(1)
        vcadd.DrawCurrentEntity()

        vcadd.SetCurrentEntity(hEntRubber + 1)
        vcadd.SetCurrentEntityPurged(1)
        vcadd.DrawCurrentEntity()
    End If

    '   We are done so clear the alert events so Visual CADD
    '   no longer sends the alerts.
    vcadd.ClearAlertEvent()

    '   Restore the saved settings so the user settings are as 
    '   they began.
    vcadd.RestoreSettings()
End Sub
 
Function Atan2(y, x)
    '   Atan2 for VBScript
    If (x > 0.0) Then
        Atan2 = Atn(y / x)
    ElseIf (x < 0.0) Then
        Atan2 = Sgn(y) * (Pi - Atn(Abs(y / x)))
    ElseIf (y = 0.0) Then
        Atan2 = 0.0
    Else
        Atan2 = Sgn(y) * Pi / 2.0
    End If
End Function
