' VBScript to Parse Hatch/Fill Boundary
'   lines starting with an apostrophe are comments

Public Const HATCH2D = 14

' create Visual CADD object
set vcadd = CreateObject("VisualCADD.Application.7")

'   used for messages
Dim sz

'   Retrieve the ENTITYHANDLE for the hatch or fill entity:
'       In order to access the boundaries of a hatch or fill 
'       entity, an application needs to retrieve the handle 
'       for the entity.  This handle is usually retrieved 
'       with VCGetCurrEntityHandle after utilizing another 
'       database parsing routine.  Please see the rest of the 
'       database parsing section for details on the methods 
'       available for accessing entity information.  For
'       example, we will use the first hatch entity in the 
'       database: 

'   start at the first entity and look for first hatch
Dim iKind
iKind = vcadd.FirstEntity()

Do While ((vcadd.GetErrorCode() = 0) And (iKind <> HATCH2D))
    iKind = vcadd.NextEntity()
Loop

'   setup a counter
Dim iCount
iCount = 0

If (vcadd.GetErrorCode() = 0) Then
    '   found hatch, save the entity handle

    Dim hEnt
    hEnt = vcadd.GetCurrentEntityHandle()

    '   Set a parsing loop and determine the number of contours 
    '   defining the boundary:
    '       The hatch or fill boundary can be defined by 
    '       multiple paths or contours.  The number of 
    '       contours can be retrieved with the API routine 
    '       VCGetEntityContourCount.  The following example 
    '       returns a count for the number of contours 
    '       defining the entity.  It assumes a known 
    '       entity handle obtained through other database 
    '       parsing methods:

    Dim iContCount
    iContCount = vcadd.GetEntityContourCount(hEnt)

    '   do a parsing loop through all the contours
    For iContIndex = 0 To iContCount - 1

        '   determine the number of sub-entities defining 
        '   each contour
        Dim iSubCount
        iSubCount = vcadd.GetEntitySubEntityCount(hEnt, iContIndex)

        '   do a parsing loop through all the sub-entities of 
        '   each contour
        For iEntIndex = 0 To iSubCount - 1
            '   Your tool would do whatever processing is 
            '   required.  For example, if you needed to 
            '   process the sub-entities, you may make the 
            '   sub-entity the current entity by calling 
            '   VCSetCurrentEntitySubEntity.  For example, 
            '   count the total sub-entities:
            iCount = iCount + 1
        Next
    Next
End If

sz = "First hatch total sub-entity count: " + CStr(iCount)
MsgBox sz, vbInformation, "API Help Sample"

' release Visual CADD object
set vcadd = Nothing
