' VBScript to Retrieve User Data
'   lines starting with an apostrophe are comments

' create Visual CADD object
set vcadd = CreateObject("VisualCADD.Application.7")

'   Set the required unique user data name in the application
'   initialization routine:
'       In order to access user data for each entity and the 
'       drawing header, a unique user data name must be 
'       specified to reference the information.  The user 
'       data name is case insensitive and can be up to 15 
'       characters long.  The following code sets a user 
'       data name of "MyData":

vcadd.SetUserDataName("MyData")

'   Retrieve the information to the current entity:
'       To access user data in the current entity, call 
'       VCSetEntityUserData.

vcadd.SetEntityUserData()

'     The current entity can be set in a number of ways 
'     using any of the Visual CADD API parsing routines or 
'     VCSetCurrentEntity.  For information see the topic 
'     Parsing the Database.  For illustration, we will use 
'     the first entity in the database with entity handle or 
'     index 0.  The user data will be retrieved from this 
'     current entity.

vcadd.SetCurrentEntity(0)

'   verify we have a current entity before proceeding
If (vcadd.GetErrorCode() = 0) Then

    '   Retrieve the user data count for the unique user 
    '   data ID:
    '       This count returns the number of valid user data 
    '       segments attached to the entity.  This 
    '       information can then used to loop through the 
    '       retrieval process.  The following code retrieves 
    '       the amount of data attached at the current 
    '       location:

    Dim iUDCount
    iUDCount = vcadd.GetCurrentEntityUserDataCount()

    '   For each index, retrieve the type of user data 
    '   information attached.  This step can be avoided if 
    '   the application remembers and accesses the 
    '   information all in the same order each time.  The 
    '   following code retrieves the user data kind at the 
    '   specified index.  Typically this would be done in a 
    '   loop statement to check all the data on the entity:

    For iIndex = 0 To iUDCount - 1
        '   If the data type is not known then typically this 
        '   would involve a statement to retrieve the user 
        '   data kind then retrieve the value of the 
        '   appropriate type.  If the application remembers 
        '   and accesses the user data the same way each 
        '   time, you would just make direct calls to the 
        '   corresponding functions for the respective index.

        Dim iUDKind
        iUDKind = vcadd.GetCurrentEntityUserDataKind(iIndex)

        Select Case iUDKind
            Case USER_BYTE
                Dim bByte
                bByte = vcadd.GetCurrentEntityUserDataByte(iIndex)
            Case USER_SHORT
                Dim nShort
                nShort = vcadd.GetCurrentEntityUserDataShort(iIndex)
            Case USER_LONG
                Dim iLong
                iLong = vcadd.GetCurrentEntityUserDataLong(iIndex)
            Case USER_DOUBLE
                Dim dDouble
                dDouble = vcadd.GetCurrentEntityUserDataDouble(iIndex)
            Case USER_FLOAT
                Dim fFloat 
                fFloat = vcadd.GetCurrentEntityUserDataFloat(iIndex)
            Case USER_CHUNK
                Dim iSize
                iSize = vcadd.GetCurEntUserDataChunkSize(iIndex)
                Dim pBytes()
                pBytes = vcadd.GetCurrentEntityUserDataChunk(iIndex)
                For i = 0 To iSize - 1
                    Dim b
                    b = pBytes(i) ' access the array
                Next
            Case USER_STRING
                Dim sz
                sz = vcadd.GetCurrentEntityUserDataString(iIndex)
        End Select
    Next
End If

' release Visual CADD object
set vcadd = Nothing
